import os
import click
from google_oauth2_desktop_flow.tokens import get_tokens

DEFAULT_CLIENT_SECRET_PATH = os.path.expanduser('~/.goauth/oauth_client_secret.json')


@click.command()
@click.option('-s', '--scopes', required=True, multiple=True, type=str)
@click.option('-c', '--client-secret-path', default=DEFAULT_CLIENT_SECRET_PATH)
def main(scopes, client_secret_path):
    credentials = get_tokens(scopes, client_secret_path)
    click.secho('Google OAuth2.0 Tokens', fg='green', bold=True, underline=True)

    click.secho('Access Token: ', fg='green', nl=False)
    click.echo(credentials.token)
    click.secho('Refresh Token: ', fg='green', nl=False)
    click.echo(credentials.refresh_token)
    click.secho('Token Uri: ', fg='green', nl=False)
    click.echo(credentials.token_uri)
    click.secho('Client Id: ', fg='green', nl=False)
    click.echo(credentials.client_id)
    click.secho('Scopes: ', fg='green', nl=False)
    click.echo(', '.join(credentials.scopes))



from setuptools import setup, find_packages

setup(
    name='google_oauth2_desktop_flow',
    version='0.0.5',
    author='Paulo Rosário',
    author_email='paulo.filip3@gmail.com',
    packages=find_packages(),
    install_requires=[
        'flask',
        'google-auth-oauthlib',
        'click'
    ],
    entry_points={
        'console_scripts': [
            'google-oauth-tokens = google_oauth2_desktop_flow.cli:main'
        ]
    }
)




def credentials_to_dict(credentials):
    return {
        'token': credentials.token,
        'refresh_token': credentials.refresh_token,
        'token_uri': credentials.token_uri,
        'client_id': credentials.client_id,
        'client_secret': credentials.client_secret,
        'scopes': credentials.scopes
    }




import os
import webbrowser
import google_auth_oauthlib
from http.server import HTTPServer, BaseHTTPRequestHandler
from urllib.parse import urlparse, parse_qs
from multiprocessing import Process, Queue

DEFAULT_CLIENT_SECRET_PATH = os.path.expanduser('~/.goauth/oauth_client_secret.json')

Q = Queue()

class RequestHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        parsed_query = parse_qs(urlparse(self.path).query)

        if not parsed_query:
            raise ValueError("Received empty URL query string.")

        code = parsed_query.get('code', None)
        if not code:
            raise ValueError("OAuth code not found in query string.")

        Q.put(parsed_query['code'][0])
        self.send_response(200)
        self.end_headers()
        self.wfile.write(b'You can close this window now!')
        self.server.shutdown()


def get_tokens(scopes, client_secret_path=DEFAULT_CLIENT_SECRET_PATH):
    server = HTTPServer(('127.0.0.1', 8081), RequestHandler)
    server = Process(target=server.serve_forever)
    server.start()

    flow = google_auth_oauthlib.flow.Flow.from_client_secrets_file(
        client_secret_path,
        scopes=scopes)
    flow.redirect_uri = 'http://127.0.0.1:8081'
    url = flow.authorization_url(access_type='offline')

    webbrowser.open(url[0])
    print(f'Open the following URL in the browser: {url[0]}')
    oauth_code = Q.get()
    server.terminate()
    flow.fetch_token(code=oauth_code)
    return flow.credentials
